mode(-1);

// Author: Bryan Gardiner
// For: Yves Lucet
// Date: Summer 2008

// Demo file: Compare the brute-force (exponential), direct
// (recursive, cubic), fast (recursive, quadratic), and fitzinf
// (quadratic) algorithms by computing the Fitzpatrick function
// of the operator f(x)=x^2/2 on a grid as m=Nx=Nx* increase.

// Functions demoed: op_fitz_brute, op_fitz_direct, op_fitz, op_fitzinf

// Result: With sufficiently large parameters, the polynomial
// functions do achieve their theoretical results:
//   Algorithm:      Complexity:
//   op_fitz_direct    O(m^4)
//   op_fitz           O(m^3)
//   op_fitzinf        O(m^2)
// As well, op_fitz_brute appears to have exponential time because
// of the rate of increase of its curve's slope.  op_fitzinf0 is not
// tested because xstar, fixed at 0, cannot be O(m).

global M;
global time;
global coefs;
global algLabels;
s=stacksize();
if (s(1) < 1e6) then
  if (msscanf(getversion(),"%*7s%f") < 5.2) then
    stacksize(1e6);
  else
    stacksize("max");
  end;
end;

algLabels = ["brute", "direct", "fitz", "inf"];

function y = ff(x), y = x .^ 2 / 2, endfunction;
function s = dff(x), s = x, endfunction;

function [logM, logTime] = computeLog(M, time)
  logM = log(M);
  [r,c] = find(time == 0);
  logTime = time;
  for i = 1:length(r)
    logTime(r(i),c(i)) = 1;
  end;
  logTime = log(logTime);
  for i = 1:length(r)
    logTime(r(i),c(i)) = -%inf;
  end;
endfunction

function coefs = doRegression(logM, logTime, algLabels)
  // Compute the regression coefficients.
  // For each algorithm, drop all times before and including the
  // last 0 time (which is -%inf in logTime).  Any non-zero readings
  // before the last 0 are too inaccurate.

  if (argn(2) < 3) then algLabels = []; end;

  for algIdx = 1:size(logTime, 2)
    algLogTime = logTime(:,algIdx);
    i = max(find(isinf(algLogTime) & sign(algLogTime) == -1)) + 1;
    j = min(find(isnan(logTime(:,algIdx))));
    if (j == []) then
      j = size(logTime, 1);
    else
      j = j - 1;
    end;
    if (i >= j) then
      coefs(algIdx) = -%inf;    // gsort works with +/-%inf but not %nan.
    else
      coef = regress(logM(i:j), logTime(i:j,algIdx));
      coefs(algIdx) = coef(2);
    end;
  end;

  if (algLabels ~= []) then
    [s,k] = gsort(coefs, "r", "i");
    l = algLabels(k)';
    for i = 1:size(logTime, 2)       // Print by decreasing efficiency
      if (s(i) == -%inf)
        printf("%s: Not enough data\n", l(i));
      else
        printf("%s: C=%f\n", l(i), s(i));
      end;
    end;
    printf("(time = a*m^C, with m=Nx=Nx* and for some a)\n");
  end;
endfunction

function plotResults(M, time, coefs, algLabels, logScale)
  if (argn(2) < 5) then logScale = %f; end;

  [s,k] = gsort(coefs, "r", "i");
  l = algLabels(k)';

  for i = 1:4
    maxIdx = max(find(~isnan(time(:,i))));
    minIdx = max(find(time(1:maxIdx,i) <= 1e-8)) + 1;
    time(1:minIdx-1,i) = time(minIdx,i);
    printf("Col %d: %d-%d\n", i, minIdx, maxIdx);
  end;
  disp(time);

  // Plot the results.
  scf();
  if (logScale) then
    plot2d(M, time(:,k), logflag="ll");
  else
    plot2d(M, time(:,k), ...
        rect=[M(1), 0, M($), max(time(:,graphUpperYLimitAlgs))]);
  end;
  p = get("hdl");
  legends(algLabels(k), 1:size(algLabels, 2), opt="ul");
  q = get("hdl");
  if (~logScale) then
    move(q, [0.14, 0]);
  end;
  thickness = [1;2;1;1;1];
  line_style = [1;1;2;3;4];
  for i = 1:size(algLabels, 2)
    p.children(i).thickness = thickness(i);
    p.children(i).line_style = line_style(i);
    q.children(2*i).thickness = thickness(i);
    q.children(2*i).line_style = line_style(i);
  end;
  if (~logScale) then
    p.parent.sub_ticks(1) = 0;
  end;
  if (logScale) then
    xtitle("Comparison of Fitzpatrick Algorithms", "log(m = Nx = Nx*)", "log(CPU seconds)");
  else
    xtitle("Comparison of Fitzpatrick Algorithms", "m = Nx = Nx*", "CPU seconds");
  end;
endfunction

doCalc = %t;
doPlot = %t;

select x_choose([ ...
        "Quick (small parameters)", ...
        "Thorough (slower, better curves)", ...
        "More thorough", ...
        "Custom", ...
        "Just plot"], ...
    "What kind of plot would you like to make?")
case 0 then // Cancel
  doCalc = %f;
  doPlot = %f;

case 1 then // Quick
  M = [2:20];
  directEnd = 4;
  recEnd = 14;
  lltEnd = M($);
  graphUpperYLimitAlgs = 3:4;

case 2 then // Thorough
  M = [2:30];
  directEnd = 5;
  recEnd = 18;
  lltEnd = M($);
  graphUpperYLimitAlgs = 3:4;

case 3 then // More thorough
  M = [2:60];
  directEnd = 5;
  recEnd = 32;
  lltEnd = M($);
  graphUpperYLimitAlgs = 3:4;

case 4 then // Custom
  params = x_mdialog("Choose demo parameters", ...
      ["Lower m", "Upper m", "Final m for fitz_direct", ...
          "Final m for fitz_rec", "Final m for fitz_llt", ...
          "Limit y-max to which algorithm? [1..4]"], ...
      ["2", "20", "4", "14", "20", "3"]);
  if (params == []) then
    run = %f;
  else
    params = evstr(params);
    M = params(1):params(2);
    directEnd = params(3);
    recEnd = params(4);
    lltEnd = params(5);
    graphUpperYLimitAlgs = min(max(params(6), 1), 4):4;
  end;

case 5 then // Just plot
  doCalc = %f;
  graphUpperYLimitAlgs = 3:4;
end;

if (doPlot) then
  select x_choose(["Linear", "Logarithmic", "None, don''t plot"], "Choose a graph scale.")
  case 0 then
    doCalc = %f;
    doPlot = %f;

  case 1 then
    logScale = %f;

  case 2 then
    logScale = %t;
  end;
end;

// m = Nx = Nx*

if (doCalc) then
  directEndIdx = find(M == directEnd);
  recEndIdx = find(M == recEnd);
  lltEndIdx = find(M == lltEnd);

  printf("Varying m from %d to %d.\n", M(1), M($));
  Mcount = length(M);
  isEqual = %t;
  time = %inf * eye(length(M), 4);

  beginProgress();
  startTime = getdate();
  for i = 1:length(M)
    m = M(i);
    a = (0:10/(m-1):10);
    astar = ff(a);
    A1 = [a; astar];
    A2 = [a; astar; astar];
    x = a;
    xstar = (0:100/(m-1):100);

    //showProgress((i-1)/Mcount, %f, sprintf("m = %d, inf:  , fitz:   , direct:   , brute:   ", m));
    showProgress((i-1)/Mcount, %f, sprintf("m = %d", m));
    timer(); M4 = op_fitzinf(A2, x, xstar); time(i, 4) = timer();
    //showProgress((i-1)/Mcount, %f, sprintf("m = %d, inf: Ok, fitz:   , direct:   , brute:   ", m));
    showProgress((i-1)/Mcount, %f, sprintf("m = %d", m));
    if (m <= lltEnd) then
      timer(); M3 = op_fitz(A1, m+1, x, xstar); time(i, 3) = timer();
      //showProgress((i-1)/Mcount, %f, sprintf("m = %d, inf: Ok, fitz: Ok, direct:   , brute:   ", m));
      showProgress((i-1)/Mcount, %f, sprintf("m = %d", m));
      if (abs(M4 - M3) >= 1E-6) then isEqual = %f; break; end;
    end;

    if (m <= recEnd) then
      timer(); M2 = op_fitz_direct(A1, m+1, x, xstar); time(i, 2) = timer();
      //showProgress((i-1)/Mcount, %f, sprintf("m = %d, inf: Ok, fitz: Ok, direct: Ok, brute:   ", m));
      showProgress((i-1)/Mcount, %f, sprintf("m = %d", m));
      if (abs(M4 - M2) >= 1E-6) then isEqual = %f; break; end;
    end;

    if (m <= directEnd) then
      timer(); M1 = op_fitz_brute(A1, m+1, x, xstar); time(i, 1) = timer();
      //showProgress((i-1)/Mcount, %f, sprintf("m = %d, inf: Ok, fitz: Ok, direct: Ok, brute: Ok", m));
      showProgress((i-1)/Mcount, %f, sprintf("m = %d", m));
      if (abs(M4 - M1) >= 1E-6) then isEqual = %f; break; end;
    end;
  end;
  finishTime = getdate();
  totalTime = etime(finishTime, startTime);
  finishProgress();

  printf("Run in %f seconds.\n", totalTime);

  //test results
  if (~isEqual) then
    printf("Error! For i=%i not all methods returned the same result\n",i);
  end;

  [logM, logTime] = computeLog(M, time);
  coefs = doRegression(logM, logTime, algLabels);
  printf("Computation time is %f seconds.\n", sumFinites(time));
end;

if (doPlot) then
  if (~doCalc) then
    [logM, logTime] = computeLog(M, time);
    coefs = doRegression(logM, logTime, algLabels);
    printf("Computation time is %f seconds.\n", sumFinites(time));
  end;

  plotResults(M, time, coefs, algLabels, logScale);
end;
