function F = op_fitzinf(B, x, xstar)
  // Evaluate the Fitzpatrick function F[A,%inf](x, xstar) via
  // Rockafellar functions using an O(m^3) algorithm when m==Nx==Nx*.
  // The code to compute one PLQ function is the same as that in
  // plq.sci:plq_rock().
  //
  // F[A, %inf](x, xstar) = sup [a in Dom A] (a*xstar + R[A,a](x))
  // where B: x -> conv(A*x): union(i=1:m, {a(i)} cross [bminus(i), bplus(i)].
  //
  // B = [a; bminus; bplus];    // a, bminus, bplus are row vectors.

  if (size(x, 1) < size(x, 2)) then x = x'; end;
  if (size(xstar, 1) < size(xstar, 2)) then xstar = xstar'; end;

  if (~and(B(1,2:$) - B(1,1:$-1) >= 0)) then
    [a,k] = gsort(B(1,:), "c", "i");
    B = [a; B(2:$,k)];
  end;

  if (or(B(1,2:$) - B(1,1:$-1) == 0)) then
    cerror("op_fitzinf: a values are not strictly increasing.");
    F = [];
    return;
  end;

  // Is the operator not monotonically nondecreasing?
  if (~(and(B(3,:)-B(2,:) >= 0) & and(B(2,2:$)-B(3,1:$-1) >= 0))) then
    F = %inf * ones(size(x, 1), size(xstar, 1));
    return;
  end;

  //          1         2:m+1        m+2
  m = size(B, 2);
  a = [-%inf, B(1,:), %inf];
  bm = [%nan, B(2,:), %nan];
  bp = [%nan, B(3,:), %nan];
  R = [];

  // Compute k=1:m functions R[A,a_k](x), each having i=1:m terms.
  // O(m(m + Nx)) = O(m^2 + Nx)
  for k = 2:m+1                                   // O(m)
    plqf = [];
    xi = 2;
    // Precompute a sum to improve the efficiency of repeatedly
    // computing Rockafellar functions.
    Sminus = sum((a(2:k-1) - a(3:k)) .* bm(3:k)); // ..O(m)
    Splus = 0;

    for i = 2:m+1                                 // ..O(m)
      if (i <= k) then
        b = bm(i);
        c = -a(i)*bm(i) + Sminus;
        Sminus = Sminus - (a(i) - a(i+1))*bm(i+1);
        plqf = [plqf; a(xi), 0, b, c];
        xi = xi + 1;
      end;

      if (i >= k) then
        b = bp(i);
        c = -a(i)*bp(i) + Splus;
        Splus = Splus + (a(i+1) - a(i))*bp(i);
        plqf = [plqf; a(xi), 0, b, c];
        xi = xi + 1;
      end;
    end;

    //printf("op_fitzinf, plq function #%d:", k);
    //disp(plqf);
    R = [R; -plq_eval(plqf, x)'];                 // ..O(m + Nx)
  end;

  //disp(-R);
  F = zeros(size(x,1), size(xstar,1));
  a = B(1,:)';
  Nx = size(x, 1);
  for i = 1:Nx                           // O(Nx)
    F(i,:) = pl_lft_llt(a, R(:,i), xstar)'; // ..O(m + Nxstar)
  end;
endfunction