function F = plq_fitzinf0(B)
  // Compute F[A,%inf](., 0) = max[a] R[A,a](.)
  //                         = max[1<=k<=m] R[a_k](.)
  // where B: x -> conv(A*x): union(i=1:m, {a(i)} cross [bminus(i), bplus(i)].
  //
  // B = [a; bminus; bplus];    // a, bminus, bplus are row vectors.
  //
  // This is an optimized version of plq_fitzinf0_direct,
  // which runs in big-theta(m) time.

  if (~and(B(1,2:$) - B(1,1:$-1) >= 0)) then
    [a,k] = gsort(B(1,:), "c", "i");
    B = [a; B(2:$,k)];
  end;

  if (or(B(1,2:$) - B(1,1:$-1) == 0)) then
    cerror("op_fitzinf0: a values are not strictly increasing.");
    F = [];
    return;
  end;

  // Is the operator not monotonically nondecreasing?
  if (~(and(B(3,:)-B(2,:) >= 0) & and(B(2,2:$)-B(3,1:$-1) >= 0))) then
    F = [%inf, 0, 0, %inf];
    return;
  end;

  m = size(B, 2);
  a = [-%inf; B(1,:)'; %inf];
  bm = [-%inf; B(2,:)'; %inf];
  bp = [-%inf; B(3,:)'; %inf];

  // Todo: Use binary searches.
  j0 = max(find(bm < 0));
  j1 = max(find(bp < 0));

  S1 = sum((a(2:j0-1) - a(3:j0)) .* bm(3:j0));
  S2 = 0;

  // Compute F[A,%inf].  O(m)
  F = [];
  for i = 1:m+1
    plqx = a(i+1);    // a(i) < x < a(i+1)

    if (j0 > i) then
      // Case 2: R_{a_{j_0}}
      //printf("plq_fitzinf0, i = %d, Case 2\n", i);
      plqb = bm(i+1);
      plqc = -a(i+1)*bm(i+1) + S1;
      S1 = S1 - (a(i+1) - a(i+2)) * bm(i+2);
    else
      if (i-1 >= j1+1) then
        S2 = S2 + (a(i) - a(i-1)) * bp(i-1);
        plqc = S2;
        //printf("Sum %f, ", S2);
      else
        plqc = 0;
        //printf("No sum %f, ", S2);
      end;

      if (j0 < i) then
        // Case 1: R_{a_{j_1 + 1}}
        //printf("plq_fitzinf0, i = %d, Case 1\n", i);
        plqb = bp(i);
        plqc = plqc - a(i)*bp(i);
      elseif (bp(i) == bm(i+1) | i == m+1) then
        // Case 3: R_{a_{j_1 + 1}}
        //printf("plq_fitzinf0, i = %d, Case 3\n", i);
        plqb = bp(i);
        plqc = plqc - a(i)*bp(i);
      else
        // Case 4: R_{a_{j_1 + 1}} then R_{a_{i+1}}
        //printf("plq_fitzinf0, i = %d, Case 4\n", i);
        if (i ~= 1) then
          xbar = (a(i)*bp(i) - a(i+1)*bm(i+1)) / (bp(i) - bm(i+1));
          plqb = bp(i);
          plqc = plqc - a(i)*bp(i);
          F = [F; xbar, 0, plqb, plqc];
        end;

        plqb = bm(i+1);
        plqc = -a(i+1)*bm(i+1);
      end;
    end;

    F = [F; plqx, 0, plqb, plqc];
  end;

  F = plq_clean(F);
endfunction