// Author:   Jeff Dicker
// For:      Dr. Yves Lucet
// Whatsit:  Unit tests for _pl_fusion, _pl_fusionsci
// 	     Comparison of _pl_fusionsci (sort using buip-in functions) with _pl_fusion (merge sort)
//
// Test-script for the Linear-time Legendre Transform (Yves Lucet - 09/05/97)
//// ported to Octave (Yves Lucet - 11/18/04)
//// ported to Scilab w/ Unit Tests (Jeff Dicker - 05/09/05)
///////////////////////////////////////////////////////////////////////////

// Beware that taking random numbers affects the computation time.
// Indeed if s_1<...<s_m<c_1<...<c_p, both functions are very fast
// (since they do not look at all slopes S). Unfortunately, this case gives
// poor numerical accuracy.
///////////////////////////////////////////////////////////////////////////

//_pl_fusion and _pl_fusionsci are internal functions that should not be directly manipulated
//the code below is for unit testing purposes only
mode(-1);
__test_set = "Fusion";

//unit tests
function b = testFusionAllTouchFirst()
	C = [8 9 10 12]';
	S = [1 2 3 4 5]';
	result = _pl_fusion(C, S);
	b = isequal(result, [1 1 1 1 1]');
endfunction

function b = testFusionNoneTouch()
	C = [-4 -2 0 1 4]';
	S = [6 8 9 10 12]';
	result = _pl_fusion(C, S);
	// it should return the index 6 size(S, 2) times, because
	// none of the points will touch
	b = isequal(result, [6 6 6 6 6]');
endfunction

function b = testFusionMiddleOne()
	C = [-4 -2 0 2 4]';
	S = [-3 -1 1 3]';
	result = _pl_fusion(C, S);
	// S(1) = -3 <= C(2) = -2     [2]
	// S(2) = -1 <= C(3) = 0      [3]
	// S(3) = 1  <= C(4) = 2      [4]
	// S(4) = 3  <= C(5) = 4      [5]
	b = isequal(result, [2 3 4 5]');
endfunction

function b = testFusionMiddleTwo()
	C = [-8 4 7 9 36]';
	S = [5 6 13]';
	result = _pl_fusion(C, S);
	// S(1) = 5  <= C(3) = 7      [3]
	// S(2) = 6  <= C(3) = 7      [3]
	// S(3) = 13 <= C(5) = 36     [5]
	b = isequal(result, [3 3 5]');
endfunction

function b = testFusionAllFirstsci()
	C = [8 9 10 12]';
	S = [1 2 3 4 5]';
	result = _pl_fusionsci(C, S);
	b = isequal(result, [1 1 1 1 1]');
endfunction

function b = testFusionNoneTouchsci()
	C = [-4 -2 0 1 4]';
	S = [6 8 9 10 12]';
	result = _pl_fusionsci(C, S);
	// it should return the index 6 size(S, 2) times, because
	// none of the points will touch
	b = isequal(result, [6 6 6 6 6]');
endfunction

function b = testFusionMiddleOnesci()
	C = [-4 -2 0 2 4]';
	S = [-3 -1 1 3]';
	result = _pl_fusionsci(C, S);
	// S(1) = -3 <= C(2) = -2     [2]
	// S(2) = -1 <= C(3) = 0      [3]
	// S(3) = 1  <= C(4) = 2      [4]
	// S(4) = 3  <= C(5) = 4      [5]
	b = isequal(result, [2 3 4 5]');
endfunction

function b = testFusionMiddleTwosci()
	C = [-8 4 7 9 36]';
	S = [5 6 13]';
	result = _pl_fusionsci(C, S);
	// S(1) = 5  <= C(3) = 7      [3]
	// S(2) = 6  <= C(3) = 7      [3]
	// S(3) = 13 <= C(5) = 36     [5]
	b = isequal(result, [3 3 5]');
endfunction

function b = testAllFusionsAreTheSame()
	X=[-5:0.5:5]';
	Y=X.^2;
	C=(Y(2:size(Y,1))-Y(1:size(Y,1)-1))./(X(2:size(X,1))-X(1:size(X,1)-1));
	S=C-0.25*ones(size(C, 1));
	fSsci = _pl_fusionsci(C, S);
	fSfus = _pl_fusion(C, S);
	b = isequal(fSsci, fSfus);
endfunction

function b = testSingleSFusions()
	X = (-5:0.5:5)';
	Y = X.^4;
	C=(Y(2:size(Y,1))-Y(1:size(Y,1)-1))./(X(2:size(X,1))-X(1:size(X,1)-1));
	S=C-0.25*ones(size(C, 1));
	for s=S'
	  fSsci = _pl_fusionsci(C, s);
	  fSfus = _pl_fusion(C, s);
	  b = (length(fSsci) == 1 & isequal(fSsci, fSfus));
	  if (~b) then 
        //index returned is different. Does it point to the same value?
        //i.e. sorting [1 1 2] can return indices in order [1 2 3] or [2 1 3]
        //both are valid (the first one is a stable sort)        
        ind=find(s==C);
        Ok = and([ind==10,fSsci==11]);
        if ~(Ok) then break; end;
      end      
    end;
endfunction

function b = testEmptySFusions()
	X=[-5:0.5:5]';
	Y=X.^2;
	C=(Y(2:size(Y,1))-Y(1:size(Y,1)-1))./(X(2:size(X,1))-X(1:size(X,1)-1));
	S=[];
	fSsci = _pl_fusionsci(C, S);
	fSfus = _pl_fusion(C, S);
	b = isequal([], fSsci, fSfus);
endfunction


function b = runTestFile
	b = %t;	
	r = testWrapper(testAllFusionsAreTheSame, "testAllFusionsAreTheSame");
	b = checkForFail(r, b);
	r = testWrapper(testFusionAllTouchFirst, "testFusionAllTouchFirst");
	b = checkForFail(r, b);
	r = testWrapper(testFusionNoneTouch, "testFusionNoneTouch");
	b = checkForFail(r, b);
	r = testWrapper(testFusionMiddleOne, "testFusionMiddleOne");
	b = checkForFail(r, b);
	r = testWrapper(testFusionMiddleTwo, "testFusionMiddleTwo");
	b = checkForFail(r, b);
	r = testWrapper(testFusionAllFirstsci, "testFusionAllFirstsci");
	b = checkForFail(r, b);
	r = testWrapper(testFusionNoneTouchsci, "testFusionNoneTouchsci");
	b = checkForFail(r, b);
	r = testWrapper(testFusionMiddleOnesci, "testFusionMiddleOnesci");
	b = checkForFail(r, b);
	r = testWrapper(testFusionMiddleTwosci, "testFusionMiddleTwosci");
	b = checkForFail(r, b);
	r = testWrapper(testSingleSFusions, "testSingleSFusions");
	b = checkForFail(r, b);
	r = testWrapper(testEmptySFusions, "testEmptySFusions");
	b = checkForFail(r, b);
endfunction
